(function($) {
	window.Serifu = {
		allWraps: [],
		files: [],
		attachmentId: 0,
		isProcessing: false,
		replaceType: 'url',
		init: function() {
			Serifu.initWraps();
			Serifu.bindReplace();
		},
		toggleType: function ($wrap, $obj) {
			$wrap.find('label').removeClass('selected');
			$obj.parent().addClass('selected');
			if ($obj.val() === '' || $obj.val() === 'url') {
				Serifu.replaceType = 'url';
				$wrap.find('.js-serifu-replace-type-url').removeClass('off');
				$wrap.find('.js-serifu-replace-type-upload').addClass('off');
			} else {
				Serifu.replaceType = 'upload';
				$wrap.find('.js-serifu-replace-type-url').addClass('off');
				$wrap.find('.js-serifu-replace-type-upload').removeClass('off');
			}
		},
		bindTypeToggles: function ($wrap) {
			var $type = $wrap.find('input[name="serifu_replace_type"]');
			if ($type.length) {

				$type.bind('click.serifu', function() {
					Serifu.toggleType($wrap, $(this));
				});
				$('#serifu_replace_type_' + serifuSettings.defaultType).trigger('click.serifu');
			}
		},
		initWraps: function () {
			Serifu.allWraps = $('.js-serifu-image-fetch-wrap');
		},
		preventDefault: function (event) {
			// Stop stuff happening.
			event.stopPropagation();

			// Totally stop stuff happening.
			event.preventDefault();
		},
		unbindClicks: function () {
			Serifu.allWraps.each(function() {
				var $wrap = $(this);

				// Toggles for the types.
				var $type = $wrap.find('input[name="serifu_replace_type"]');
				if ($type.length) {
					$type.unbind('click.serifu');
				}

				// Fetch image action.
				var $button = $wrap.find('.js-serifu-image-fetch');
				if ($button.length) {
					$button.unbind('click.serifu');
				}

				var $button2 = $wrap.find('.js-serifu-image-upload');
				if ($button2.length) {
					$button2.unbind('click.serifu');
				}
			});
		},
		bindReplaceButtons: function($wrap) {
			// Fetch image action.
			var $button = $wrap.find('a.js-serifu-image-fetch');
			if ($button.length) {
				$button.on('click', function(event) {
					Serifu.preventDefault(event);
				});

				$button.bind('click.serifu', function(event) {
					Serifu.preventDefault(event);
					var new_url = $wrap.find('input.js-serifu-image-fetch').val();
					if (new_url === '') {
						Serifu.exposeError($wrap, null, serifuSettings.emptyUrl);
					} else {
						if (Serifu.isProcessing === false) {
							Serifu.isProcessing = true;
							$wrap.addClass('processing');

							$.ajax({
								type: 'POST',
								url: serifuSettings.ajaxUrl,
								dataType: 'json',
								data: {
									action: 'sisanu_image_replacement_from_url',
									old_image_id: $button.data('attachment-id'),
									new_image_url: new_url,
								},
								cache: false,
							}).done(function (response) {
								Serifu.processResponse(response, $wrap, $button);
							});
						}
					}
				});
			}
		},
		bindUploadButtons: function($wrap) {
			// Upload image action.
			var $button2 = $wrap.find('a.js-serifu-image-upload');
			if ($button2.length) {
				$button2.on('click', function(event) {
					Serifu.preventDefault(event);
				});

				$button2.bind('click.serifu', function(event) {
					Serifu.preventDefault(event);
					if (Serifu.isProcessing === false) {

						$('#js-serifu-image-upload-frm').remove();
						$('body').append('<form id="js-serifu-image-upload-frm" method="post" enctype="multipart/form-data"></form>');
						var $theform = $('#js-serifu-image-upload-frm');

						$wrap.find('input[type="file"]').each(function(){
							$(this).clone().appendTo($theform);
						});

						Serifu.attachmentId = $button2.data('attachment-id');
						$theform.on('submit', {'$button': $button2, '$wrap': $wrap}, Serifu.uploadFiles);
						$theform.trigger('submit');
					}
				});

			}
		},
		bindReplace: function () {
			Serifu.allWraps.each(function() {
				var $wrap = $(this);

				// Toggles for the types.
				Serifu.bindTypeToggles($wrap);

				// Replace images actions.
				Serifu.bindReplaceButtons($wrap);

				// Bind file select changed.
				$wrap.find('input[type="file"]').on('change', {'$label': $wrap.find('.js-serifu-image-upload-label')}, Serifu.prepareUpload);

				// Upload image action.
				Serifu.bindUploadButtons($wrap);
			});
		},
		prepareUpload: function (event) {
			Serifu.preventDefault(event);
			Serifu.files = event.target.files;
			// Notify the selection.
			event.data.$label.html(event.target.files[0].name);
		},
		uploadFiles: function (event) {
			Serifu.preventDefault(event);
			var isSelected = false;

			// Create a formdata object and add the files
			var frmData = new FormData();
			$.each(Serifu.files, function(key, value) {
				frmData.append(key, value);
				if (value !== '') {
					isSelected = true;
				}
			});
			if (isSelected === false) {
				Serifu.exposeError(event.data.$wrap, null, serifuSettings.emptyFile);
			} else {
				frmData.append('action', 'sisanu_image_replacement_from_upload');
				frmData.append('old_image_id', event.data.$button.data('attachment-id'));

			   	event.data.$wrap.addClass('processing');
			   	Serifu.isProcessing = true;
				$.ajax({
					type: 'POST',
					url: serifuSettings.ajaxUrl,
					dataType: 'json',
					data: frmData,
					cache: false,
					processData: false, // Don't process the files.
					contentType: false, // Set content type to false as jQuery will tell the server its a query string request.
				}).done(function (response) {
					Serifu.processResponse(response, event.data.$wrap, event.data.$button);
				});
			}
		},
		exposeError: function($wrap, $responseWrap, error) {
			if ($responseWrap === null) {
				var $responseWrap = $wrap.find('.js-serifu-image-processed-response');
			}
			$responseWrap.addClass('error');
			$responseWrap.html(error);
		},
		processResponse: function(response, $wrap, $button) {
			var $responseWrap = $wrap.find('.js-serifu-image-processed-response');
			if (response !== null && response.url !== '') {
				$responseWrap.removeClass('error');
				$responseWrap.html('');
				$('#set-post-thumbnail img').attr('src', response.url);
				$('#set-post-thumbnail img').attr('srcset', response.srcset);
				$('#thumbnail-head-' + $button.data('attachment-id') + ' .thumbnail').attr('src', response.url);
			} else {
				if (Serifu.replaceType === 'url') {
					Serifu.exposeError($wrap, $responseWrap, serifuSettings.failedFetch);
				} else {
					Serifu.exposeError($wrap, $responseWrap, serifuSettings.failedUpload);
				}
			}
			$wrap.removeClass('processing');
			Serifu.isProcessing = false;

			// Reset the files values.
			$wrap.find('input[type="file"]').each(function () {
				$(this).parent().find('.js-serifu-image-upload-label').html(serifuSettings.noFile);
			});
		}
	};

	$(document).ready(function() {
		Serifu.init();
		if (wp.media) {
			wp.media.view.Modal.prototype.on('close', function() {
				setTimeout( function() {
					// Late init after modal close, to have time to bind new DOM elements.
					Serifu.init();
				}, 2000);
			});
		}
	});

})(jQuery);
